/*
 * helper.c - utility functions
 *
 * Copyright (C) 2024 iopsys Software Solutions AB. All rights reserved.
 *
 * Author: Suvendhu Hansa <suvendhu.hansa@iopsys.eu>
 *
 * See LICENSE file for license related information.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "helper.h"

static void ubus_call(struct ubus_context *ctx, const char *obj, const char *method,
			struct blob_attr *msg, ubus_data_handler_t cb, void *value)
{
	uint32_t id;

	if (ctx == NULL || obj == NULL)
		return;

	if (!ubus_lookup_id(ctx, obj, &id)) {
		ubus_invoke(ctx, id, method, msg, cb, value, 5000);
	}
}

void invoke_ethernet_hotplug(struct ubus_context *ctx, char *ifname, char *link)
{
	if (ctx == NULL || ifname == NULL || link == NULL)
		return;

	if (strlen(ifname) == 0 || strlen(link) == 0)
		return;

	char env_port[56] = {0};
	char env_link[56] = {0};
	struct blob_buf b = {0};

	snprintf(env_port, sizeof(env_port), "PORT=%s", ifname);
	snprintf(env_link, sizeof(env_link), "LINK=%s", link);

	memset(&b, 0, sizeof(struct blob_buf));
	blob_buf_init(&b, 0);

	void *arr = blobmsg_open_array(&b, "env");
	blobmsg_add_string(&b, NULL, env_port);
	blobmsg_add_string(&b, NULL, env_link);
	blobmsg_close_array(&b, arr);

	ubus_call(ctx, "hotplug.ethernet", "call", b.head, NULL, NULL);

	blob_buf_free(&b);
}

int ethmngr_clear_stats(char *ifname)
{
	if (ifname == NULL)
		return -1;

	return eth_clear_stats(ifname);
}

int ethmngr_get_rmon_stats(char *ifname, struct eth_rmon_stats *rmon, int queueid, uint16_t vlan)
{
	if (ifname == NULL || rmon == NULL)
		return -1;

	if (strlen(ifname) == 0)
		return -1;

	memset(rmon, 0, sizeof(struct eth_rmon_stats));
	rmon->txq = queueid;
	rmon->vlanid = vlan;

	int ret = eth_get_rmon_stats(ifname, rmon);
	if (ret != 0) {
		BBF_DEBUG("ethmngr: failed to get rmonstats!");
		return -1;
	}

	return 0;
}

int ethmngr_get_if_stats(char *ifname, struct eth_stats *s)
{
	if (ifname == NULL || s == NULL)
		return -1;

	if (strlen(ifname) == 0)
		return -1;

	memset(s, 0, sizeof(struct eth_stats));
	int ret = eth_get_stats(ifname, s);
	if (ret != 0) {
		BBF_DEBUG("ethmngr: failed to get interface stats!");
		return -1;
	}

	return 0;
}
